package ru.skipy.tests.ui;

import ru.skipy.tests.ui.component.graph.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * FunctionGraphTest - several graph painting techniques demo
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 08.10.2010
 */
public class FunctionGraphTest extends JFrame {

    FunctionGraphTest(final AbstractFunctionGraphPanel panel) {
        super("Function Graph test");
        JPanel cp = new JPanel(new BorderLayout(5, 5));
        cp.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        setContentPane(cp);
        cp.add(panel, BorderLayout.CENTER);
        final JComboBox graphs = new JComboBox(new Object[]{new Gauss(), new Exp(), new Atan(), new Sin()});
        cp.add(graphs, BorderLayout.SOUTH);
        graphs.setSelectedIndex(-1);
        graphs.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    panel.setGraphDescriptor((GraphDescriptor) graphs.getSelectedItem());
                    panel.repaint();
                }
            }
        });
        graphs.setSelectedIndex(0);
        setSize(500, 400);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setLocationRelativeTo(null);
        RepaintManager.currentManager(this).setDoubleBufferingEnabled(false);
    }

    public static void main(String[] args) {
        String panelType = System.getProperty("panelType");
        PanelType type = PanelType.line;
        if (panelType != null && panelType.length() > 0) {
            try {
                type = PanelType.valueOf(panelType);
            } catch (IllegalArgumentException ex) {
                System.err.println("Panel type is not supported: " + panelType);
            }
        }
        System.out.println("Using panel of type: " + type.getPanelName());
        new FunctionGraphTest(type.createPanel()).setVisible(true);
    }

    /**
     * Panel types factory
     */
    static enum PanelType {

        /**
         * direct point-to-point painting
         */
        line {
            @Override
            AbstractFunctionGraphPanel createPanel() {
                return new FunctionGraphPanelLine(null, Color.red);
            }
            @Override
            String getPanelName() {
                return "FunctionGraphPanelLine";
            }},
        /**
         * GeneralPath-based painting
         */
        gp {
            @Override
            AbstractFunctionGraphPanel createPanel() {
                return new FunctionGraphPanelGP(null, Color.red);
            }
            @Override
            String getPanelName() {
                return "FunctionGraphPanelGP";
            }},
        /**
         * General path with double buffering
         */
        gpdb {
            @Override
            AbstractFunctionGraphPanel createPanel() {
                return new FunctionGraphPanelGPDB(null, Color.red);
            }
            @Override
            String getPanelName() {
                return "FunctionGraphPanelGPDB";
            }};

        abstract AbstractFunctionGraphPanel createPanel();

        abstract String getPanelName();

    }

    /**
     * Gauss distribution function implementation (<code>f(x)=5*e<sup>-0.1*x<sup>2</sup></sup></code>)
     */
    static class Gauss extends GraphDescriptor {

        Gauss() {
            super(-10, -0.5, 10, 5.5);
        }

        @Override
        public double getValue(double x) {
            return 5 * Math.exp(-x * x * 0.1);
        }

        @Override
        public String getDescription() {
            return "5 * exp(-0.1 * x * x) on X[-10,10] & Y[-0.5, 5.5]";
        }
    }

    /**
     * Exponent function implementation (<code>f(x)=2*e<sup>-0.1*x</sup></code>)
     */
    static class Exp extends GraphDescriptor {

        Exp() {
            super(-20, -0.5, 10, 5.5);
        }

        @Override
        public double getValue(double x) {
            return 2 * Math.exp(x * 0.1);
        }

        @Override
        public String getDescription() {
            return "2 * exp(0.1 * x) on X[-20,10] & Y[-0.5, 5.5]";
        }
    }

    /**
     * arctg function implementation (<code>f(x)=arctg(x)</code>)
     */
    static class Atan extends GraphDescriptor {

        Atan() {
            super(-10, -1.6, 10, 1.6);
        }

        @Override
        public double getValue(double x) {
            return Math.atan(x);
        }

        @Override
        public String getDescription() {
            return "arctg(x) on X[-10,10] & Y[-1.6, 1.6]";
        }
    }

    /**
     * sin function implementation (<code>f(x)=sin(x)</code>)
     */
    static class Sin extends GraphDescriptor {

        Sin() {
            super(-10, -1.1, 10, 1.1);
        }

        @Override
        public double getValue(double x) {
            return Math.sin(x);
        }

        @Override
        public String getDescription() {
            return "sin(x) on X[-10,10] & Y[-1.1, 1.1]";
        }
    }
}
