package ru.skipy.tests.ui.delegate;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedList;

/**
 * ComponentPanel
 * <p/>
 * Simple component panel, which delegates all painting operations to renderer object
 *
 * @author Eugene Matyushkin aka Skipy
 * @version $Id: $
 * @since 20.12.2010
 */
public class ComponentPanel extends JPanel {

    /**
     * Half size of the selection mark
     */
    private static final int HALF_MARK = 3;

    /**
     * Renderer component
     */
    private JComponent delegate;

    /**
     * component selecton state
     */
    private boolean selected = false;

    /**
     * component selection listeners
     */
    private java.util.List<ComponentSelectionListener> listeners;

    /**
     * creates component based on the renderer passed
     *
     * @param delegate renderer object to use
     */
    public ComponentPanel(JComponent delegate) {
        super(null);
        this.delegate = delegate;
        setOpaque(false);
        listeners = new LinkedList<ComponentSelectionListener>();
        MouseHandler mh = new MouseHandler();
        addMouseListener(mh);
        addMouseMotionListener(mh);
        addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                if (ComponentPanel.this.delegate != null)
                    ComponentPanel.this.delegate.setSize(getSize());
            }
        });
    }

    /**
     * Paints component and selection state
     *
     * @param g graphics context
     */
    @Override
    protected void paintComponent(Graphics g) {
        if (delegate != null)
            delegate.paint(g);
        if (selected) {
            paintSelectionMarks(g);
        }
    }

    /**
     * Sets selection state
     *
     * @param selected state to set
     */
    void setSelected(boolean selected) {
        if (this.selected == selected)
            return;
        this.selected = selected;
        if (selected)
            fireSelectionEvent();
    }

    /**
     * fires selection event
     */
    private void fireSelectionEvent() {
        for (ComponentSelectionListener listener : listeners) {
            listener.componentSelected(this);
        }
    }

    /**
     * Paints selection marks
     *
     * @param g Graphics context
     */
    private void paintSelectionMarks(Graphics g) {
        g.setColor(Color.black);
        g.fillRect(0, 0, HALF_MARK, HALF_MARK);
        g.fillRect(getWidth() - HALF_MARK, 0, HALF_MARK, HALF_MARK);
        g.fillRect(0, getHeight() - HALF_MARK, HALF_MARK, HALF_MARK);
        g.fillRect(getWidth() - HALF_MARK, getHeight() - HALF_MARK, HALF_MARK, HALF_MARK);
        g.fillRect(getWidth() / 2 - HALF_MARK, 0, HALF_MARK * 2, HALF_MARK);
        g.fillRect(getWidth() / 2 - HALF_MARK, getHeight() - HALF_MARK, HALF_MARK * 2, HALF_MARK);
        g.fillRect(0, getHeight() / 2 - HALF_MARK, HALF_MARK, HALF_MARK * 2);
        g.fillRect(getWidth() - HALF_MARK, getHeight() / 2 - HALF_MARK, HALF_MARK, HALF_MARK * 2);
    }

    public void addComponentSelectionListener(ComponentSelectionListener listener) {
        listeners.add(listener);
    }

    /**
     * Mouse handler - to drag component. Resize can be implemented in this class also
     */
    private class MouseHandler extends MouseAdapter {

        /**
         * x of mouse pointer displacement from top-left corner
         */
        private int dx = 0;

        /**
         * y of mouse pointer displacement from top-left corner
         */
        private int dy = 0;

        @Override
        public void mousePressed(MouseEvent e) {
            setSelected(true);
            dx = e.getX();
            dy = e.getY();
        }

        @Override
        public void mouseDragged(MouseEvent e) {
            setLocation(getX() + e.getX() - dx, getY() + e.getY() - dy);
        }
    }

}
